@@ -3,6 +3,28 @@
 ## As an S3 class, it has no formal definition.
 ## ****************************************************************************
 
+#' Shortcut to provide functions to the S3 class "Kriging"
+#' @param nk A pointer to a C++ object of class "Kriging"
+#' @return An object of class "Kriging" with methods to access and manipulate the data
+classKriging <- function(nk) {
+    class(nk) <- "Kriging"
+    # This will allow to call methods (like in Python/Matlab/Octave) using `k$m(...)` as well as R-style `m(k, ...)`.
+    for (f in c('as.km','as.list','copy','fit','save',
+    'covMat','leaveOneOut','leaveOneOutFun','leaveOneOutVec',
+    'logLikelihood','logLikelihoodFun','logMargPost','logMargPostFun',
+    'predict','print','show','simulate','update', 'update_simulate')) {
+        eval(parse(text=paste0(
+            "nk$", f, " <- function(...) ", f, "(nk,...)"
+            )))
+    }
+    # This will allow to access kriging data/props using `k$d()`
+    for (d in c('kernel','optim','objective','X','centerX','scaleX','y','centerY','scaleY','regmodel','F','T','M','z','beta','is_beta_estim','theta','is_theta_estim','sigma2','is_sigma2_estim')) {
+        eval(parse(text=paste0(
+            "nk$", d, " <- function() kriging_", d, "(nk)"
+            )))
+    }
+    nk
+}
 
 #' Create an object with S3 class \code{"Kriging"} using
 #' the \pkg{libKriging} library.
@@ -11,34 +33,29 @@
 #' are estimated thanks to the optimization of a criterion given by
 #' \code{objective}, using the method given in \code{optim}.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param y Numeric vector of response values.
-#'
 #' @param X Numeric matrix of input design.
-#'
 #' @param kernel Character defining the covariance model:
 #'     \code{"exp"}, \code{"gauss"}, \code{"matern3_2"}, \code{"matern5_2"}.
-#'
-#' @param regmodel Universal Kriging linear trend.
-#'
+#' @param regmodel Universal Kriging linear trend: \code{"constant"}, 
+#'     \code{"linear"}, \code{"interactive"}, \code{"quadratic"}.
 #' @param normalize Logical. If \code{TRUE} both the input matrix
 #'     \code{X} and the response \code{y} in normalized to take
 #'     values in the interval \eqn{[0, 1]}.
-#'
 #' @param optim Character giving the Optimization method used to fit
 #'     hyper-parameters. Possible values are: \code{"BFGS"},
 #'     \code{"Newton"} and \code{"none"}, the later simply keeping
 #'     the values given in \code{parameters}. The method
-#'     \code{"BFGS"} uses the gradient of the objective. The method
+#'     \code{"BFGS"} uses the gradient of the objective 
+#'     (note that \code{"BGFS10"} means 10 multi-start of BFGS). The method
 #'     \code{"Newton"} uses both the gradient and the Hessian of the
 #'     objective.
-#'
 #' @param objective Character giving the objective function to
 #'     optimize. Possible values are: \code{"LL"} for the
 #'     Log-Likelihood, \code{"LOO"} for the Leave-One-Out sum of
 #'     squares and \code{"LMP"} for the Log-Marginal Posterior.
-#'
 #' @param parameters Initial values for the hyper-parameters. When
 #'     provided this must be named list with elements \code{"sigma2"}
 #'     and \code{"theta"} containing the initial value(s) for the
@@ -65,7 +82,7 @@
 #' print(k)
 #'
 #' x <- as.matrix(seq(from = 0, to = 1, length.out = 101))
-#' p <- predict(k, x = x, stdev = TRUE, cov = FALSE)
+#' p <- predict(k, x = x, return_stdev = TRUE, return_cov = FALSE)
 #'
 #' plot(f)
 #' points(X, y)
@@ -77,7 +94,7 @@
 #'
 #' matlines(x, s, col = rgb(0, 0, 1, 0.2), type = "l", lty = 1)
 Kriging <- function(y=NULL, X=NULL, kernel=NULL,
-                    regmodel = c("constant", "linear", "interactive"),
+                    regmodel = c("constant", "linear", "interactive", "none"),
                     normalize = FALSE,
                     optim = c("BFGS", "Newton", "none"),
                     objective = c("LL", "LOO", "LMP"),
@@ -97,26 +114,13 @@ Kriging <- function(y=NULL, X=NULL, kernel=NULL,
                       optim = optim,
                       objective = objective,
                       parameters = parameters)
-    class(nk) <- "Kriging"
-    # This will allow to call methods (like in Python/Matlab/Octave) using `k$m(...)` as well as R-style `m(k, ...)`.
-    for (f in c('as.km','as.list','copy','fit','leaveOneOut','leaveOneOutFun','leaveOneOutVec','logLikelihood','logLikelihoodFun','logMargPost','logMargPostFun','predict','print','show','simulate','update')) {
-        eval(parse(text=paste0(
-            "nk$", f, " <- function(...) ", f, "(nk,...)"
-            )))
-    }
-    # This will allow to access kriging data/props using `k$d()`
-    for (d in c('kernel','optim','objective','X','centerX','scaleX','y','centerY','scaleY','regmodel','F','T','M','z','beta','is_beta_estim','theta','is_theta_estim','sigma2','is_sigma2_estim')) {
-        eval(parse(text=paste0(
-            "nk$", d, " <- function() kriging_", d, "(nk)"
-            )))
-    }
-    nk
+    return(classKriging(nk))
 }
 
 
 #' Coerce a \code{Kriging} Object into a List
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param x An object with class \code{"Kriging"}.
 #' @param ... Ignored
@@ -151,13 +155,11 @@ as.list.Kriging <- function(x, ...) {
 #' Coerce a \code{Kriging} object into the \code{"km"} class of the
 #' \pkg{DiceKriging} package.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param x An object with S3 class \code{"Kriging"}.
-#'
 #' @param .call Force the \code{call} slot to be filled in the
 #'     returned \code{km} object.
-#'
 #' @param ... Not used.
 #'
 #' @return An object of having the S4 class \code{"KM"} which extends
@@ -182,7 +184,7 @@ as.list.Kriging <- function(x, ...) {
 #' k_km <- as.km(k)
 #' print(k_km)
 as.km.Kriging <- function(x, .call = NULL, ...) {
-
+    if (length(L <- list(...)) > 0) warnOnDots(L)
     ## loadDiceKriging()
     ## if (! "DiceKriging" %in% installed.packages())
     ##     stop("DiceKriging must be installed to use its wrapper from libKriging.")
@@ -212,9 +214,19 @@ as.km.Kriging <- function(x, .call = NULL, ...) {
     model@noise.flag <- FALSE
     model@noise.var <- 0
 
-    model@case <- "LLconcentration_beta_sigma2"
+    if (m$is_sigma2_estim)
+      model@case <- "LLconcentration_beta_sigma2"
+    else 
+      model@case <- "LLconcentration_beta"
 
-    model@known.param <- "None"
+    isTrend = !m$is_beta_estim
+    isCov = !m$is_theta_estim
+    isVar = !m$is_sigma2_estim
+    if (isCov) {
+        known.covparam <- "All"
+    } else {
+        known.covparam <- "None"
+    }
     model@param.estim <- NA
     model@method <- m$objective
     model@optim.method <- m$optim
@@ -233,7 +245,17 @@ as.km.Kriging <- function(x, .call = NULL, ...) {
     covStruct <-  new("covTensorProduct", d = model@d, name = m$kernel,
                       sd2 = m$sigma2, var.names = names(data),
                       nugget = 0, nugget.flag = FALSE, nugget.estim = FALSE,
-                      known.covparam = "")
+                      known.covparam = known.covparam)
+
+    if (isTrend && isCov && isVar) {
+        model@known.param <- "All"
+    } else if ((isTrend) && ((!isCov) || (!isVar))) {
+        model@known.param <- "Trend"
+    } else if ((!isTrend) && isCov && isVar) {
+        model@known.param <- "CovAndVar"
+    } else {    # In the other cases: All parameters are estimated (at this stage)
+        model@known.param <- "None"
+    }
 
     covStruct@range.names <- "theta"
     covStruct@paramset.n <- as.integer(1)
@@ -248,7 +270,7 @@ as.km.Kriging <- function(x, .call = NULL, ...) {
 
 #' Print the content of a \code{Kriging} object.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param x A (S3) \code{Kriging} Object.
 #' @param ... Ignored.
@@ -270,7 +292,7 @@ as.km.Kriging <- function(x, .call = NULL, ...) {
 #' ## same thing
 #' k
 print.Kriging <- function(x, ...) {
-    if (length(list(...))>0) warning("Arguments ",paste0(names(list(...)),"=",list(...),collapse=",")," are ignored.")
+    if (length(L <- list(...)) > 0) warnOnDots(L)
     p = kriging_summary(x)
     cat(p)
     invisible(p)
@@ -283,40 +305,33 @@ print.Kriging <- function(x, ...) {
 #' are estimated thanks to the optimization of a criterion given by
 #' \code{objective}, using the method given in \code{optim}.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object S3 Kriging object.
-#'
 #' @param y Numeric vector of response values.
-#'
 #' @param X Numeric matrix of input design.
-#'
-#' @param regmodel Universal Kriging linear trend.
-#'
+#' @param regmodel Universal Kriging linear trend: \code{"constant"}, 
+#'     \code{"linear"}, \code{"interactive"}, \code{"quadratic"}.
 #' @param normalize Logical. If \code{TRUE} both the input matrix
 #'     \code{X} and the response \code{y} in normalized to take
 #'     values in the interval \eqn{[0, 1]}.
-#'
 #' @param optim Character giving the Optimization method used to fit
 #'     hyper-parameters. Possible values are: \code{"BFGS"},
 #'     \code{"Newton"} and \code{"none"}, the later simply keeping
 #'     the values given in \code{parameters}. The method
-#'     \code{"BFGS"} uses the gradient of the objective. The method
+#'     \code{"BFGS"} uses the gradient of the objective (note that \code{"BGFS10"} means 10 multi-start of BFGS). The method
 #'     \code{"Newton"} uses both the gradient and the Hessian of the
 #'     objective.
-#'
 #' @param objective Character giving the objective function to
 #'     optimize. Possible values are: \code{"LL"} for the
 #'     Log-Likelihood, \code{"LOO"} for the Leave-One-Out sum of
 #'     squares and \code{"LMP"} for the Log-Marginal Posterior.
-#'
 #' @param parameters Initial values for the hyper-parameters. When
 #'     provided this must be named list with elements \code{"sigma2"}
 #'     and \code{"theta"} containing the initial value(s) for the
 #'     variance and for the range parameters. If \code{theta} is a
 #'     matrix with more than one row, each row is used as a starting
 #'     point for optimization.
-#'
 #' @param ... Ignored.
 #'
 #' @return No return value. Kriging object argument is modified.
@@ -338,11 +353,12 @@ print.Kriging <- function(x, ...) {
 #' fit(k,y,X)
 #' print(k)
 fit.Kriging <- function(object, y, X,
-                    regmodel = c("constant", "linear", "interactive"),
+                    regmodel = c("constant", "linear", "interactive", "none"),
                     normalize = FALSE,
                     optim = c("BFGS", "Newton", "none"),
                     objective = c("LL", "LOO", "LMP"),
                     parameters = NULL, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
 
     regmodel <- match.arg(regmodel)
     objective <- match.arg(objective)
@@ -366,21 +382,16 @@ fit.Kriging <- function(object, y, X,
 #' stochastic process, conditional on the values at the input points
 #' used when fitting the model.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object S3 Kriging object.
-#'
 #' @param x Input points where the prediction must be computed.
-#'
-#' @param stdev \code{Logical}. If \code{TRUE} the standard deviation
+#' @param return_stdev \code{Logical}. If \code{TRUE} the standard deviation
 #'     is returned.
-#'
-#' @param cov \code{Logical}. If \code{TRUE} the covariance matrix of
+#' @param return_cov \code{Logical}. If \code{TRUE} the covariance matrix of
 #'     the predictions is returned.
-#'
-#' @param deriv \code{Logical}. If \code{TRUE} the derivatives of mean and sd
+#' @param return_deriv \code{Logical}. If \code{TRUE} the derivatives of mean and sd
 #'     of the predictions are returned.
-#'
 #' @param ... Ignored.
 #'
 #' @return A list containing the element \code{mean} and possibly
@@ -412,16 +423,12 @@ fit.Kriging <- function(object, y, X,
 #' lines(x, p$mean, col = "blue")
 #' polygon(c(x, rev(x)), c(p$mean - 2 * p$stdev, rev(p$mean + 2 * p$stdev)),
 #'  border = NA, col = rgb(0, 0, 1, 0.2))
-predict.Kriging <- function(object, x, stdev = TRUE, cov = FALSE, deriv = FALSE, ...) {
+predict.Kriging <- function(object, x, return_stdev = TRUE, return_cov = FALSE, return_deriv = FALSE, ...) {
     if (length(L <- list(...)) > 0) warnOnDots(L)
-    k <- kriging_model(object)
     ## manage the data frame case. Ideally we should then warn
     if (is.data.frame(x)) x = data.matrix(x)
-    if (!is.matrix(x)) x=matrix(x,ncol=ncol(k$X))
-    if (ncol(x) != ncol(k$X))
-        stop("Input x must have ", ncol(k$X), " columns (instead of ",
-             ncol(x), ")")
-    return(kriging_predict(object, x, stdev, cov, deriv))
+    if (!is.matrix(x)) x=matrix(x,ncol=ncol(object$X()))
+    return(kriging_predict(object, x, return_stdev, return_cov, return_deriv))
 }
 
 
@@ -431,15 +438,16 @@ predict.Kriging <- function(object, x, stdev = TRUE, cov = FALSE, deriv = FALSE,
 #' points conditional on the values at the input points used in the
 #' fit.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object S3 Kriging object.
 #' @param nsim Number of simulations to perform.
 #' @param seed Random seed used.
 #' @param x Points in model input space where to simulate.
+#' @param will_update Set to TRUE if wish to use update_simulate(...) later.
 #' @param ... Ignored.
 #'
-#' @return a matrix with \code{length(x)} rows and \code{nsim}
+#' @return a matrix with \code{nrow(x)} rows and \code{nsim}
 #'     columns containing the simulated paths at the inputs points
 #'     given in \code{x}.
 #'
@@ -470,30 +478,78 @@ predict.Kriging <- function(object, x, stdev = TRUE, cov = FALSE, deriv = FALSE,
 #' lines(x, s[ , 1], col = "blue")
 #' lines(x, s[ , 2], col = "blue")
 #' lines(x, s[ , 3], col = "blue")
-simulate.Kriging <- function(object, nsim = 1, seed = 123, x,  ...) {
+simulate.Kriging <- function(object, nsim = 1, seed = 123, x, will_update = FALSE, ...) {
     if (length(L <- list(...)) > 0) warnOnDots(L)
-    k <- kriging_model(object)
+    ## manage the data frame case. Ideally we should then warn
     if (is.data.frame(x)) x = data.matrix(x)
-    if (!is.matrix(x)) x = matrix(x, ncol = ncol(k$X))
-    if (ncol(x) != ncol(k$X))
-        stop("Input x must have ", ncol(k$X), " columns (instead of ",
-             ncol(x),")")
-    ## XXXY
+    if (!is.matrix(x)) x=matrix(x,ncol=ncol(object$X()))
     if (is.null(seed)) seed <- floor(runif(1) * 99999)
-    return(kriging_simulate(object, nsim = nsim, seed = seed, X = x))
+    return(kriging_simulate(object, nsim = nsim, seed = seed, X_n = x, will_update = will_update))
 }
 
+#' Update previous simulation of a \code{Kriging} model object.
+#'
+#' This method draws paths of the stochastic process conditional on the values at the input points used in the
+#' fit, plus the new input points and their values given as argument (knonw as 'update' points).
+#'
+#' @author Yann Richet \email{yann.richet@asnr.fr}
+#'
+#' @param object S3 Kriging object.
+#' @param y_u Numeric vector of new responses (output).
+#' @param X_u Numeric matrix of new input points.
+#' @param ... Ignored.
+#'
+#' @return a matrix with \code{nrow(x)} rows and \code{nsim}
+#'     columns containing the simulated paths at the inputs points
+#'     given in \code{x}.
+#'
+#' @method update_simulate Kriging
+#' @export
+#' 
+#' @examples
+#' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
+#' plot(f)
+#' set.seed(123)
+#' X <- as.matrix(runif(10))
+#' y <- f(X)
+#' points(X, y, col = "blue")
+#' 
+#' k <- Kriging(y, X, kernel = "matern3_2")
+#' 
+#' x <- seq(from = 0, to = 1, length.out = 101)
+#' s <- k$simulate(nsim = 3, x = x, will_update = TRUE)
+#' 
+#' lines(x, s[ , 1], col = "blue")
+#' lines(x, s[ , 2], col = "blue")
+#' lines(x, s[ , 3], col = "blue")
+#' 
+#' X_u <- as.matrix(runif(3))
+#' y_u <- f(X_u)
+#' points(X_u, y_u, col = "red")
+#' 
+#' su <- k$update_simulate(y_u, X_u)
+#' 
+#' lines(x, su[ , 1], col = "blue", lty=2)
+#' lines(x, su[ , 2], col = "blue", lty=2)
+#' lines(x, su[ , 3], col = "blue", lty=2)
+update_simulate.Kriging <- function(object, y_u, X_u, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
+    if (is.data.frame(X_u)) X_u = data.matrix(X_u)
+    if (!is.matrix(X_u)) X_u <- matrix(X_u, ncol = ncol(object$X()))
+    if (is.data.frame(y_u)) y_u = data.matrix(y_u)
+    if (!is.matrix(y_u)) y_u <- matrix(y_u, ncol = 1)
+    ## Modify 'object' in the parent environment
+    return(kriging_update_simulate(object, y_u, X_u))
+}
 
 #' Update a \code{Kriging} model object with new points
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object S3 Kriging object.
-#'
-#' @param newy Numeric vector of new responses (output).
-#'
-#' @param newX Numeric matrix of new input points.
-#'
+#' @param y_u Numeric vector of new responses (output).
+#' @param X_u Numeric matrix of new input points.
+#' @param refit Logical. If \code{TRUE} the model is refitted (default is FALSE).
 #' @param ... Ignored.
 #'
 #' @return No return value. Kriging object argument is modified.
@@ -524,41 +580,34 @@ simulate.Kriging <- function(object, nsim = 1, seed = 123, x,  ...) {
 #' polygon(c(x, rev(x)), c(p$mean - 2 * p$stdev, rev(p$mean + 2 * p$stdev)),
 #'  border = NA, col = rgb(0, 0, 1, 0.2))
 #'
-#' newX <- as.matrix(runif(3))
-#' newy <- f(newX)
-#' points(newX, newy, col = "red")
+#' X_u <- as.matrix(runif(3))
+#' y_u <- f(X_u)
+#' points(X_u, y_u, col = "red")
 #'
 #' ## change the content of the object 'k'
-#' update(k, newy, newX)
+#' update(k, y_u, X_u)
 #'
-#' x <- seq(from = 0, to = 1, length.out = 101)
+#' ## include design points to see interpolation
+#' x <- sort(c(X,X_u,seq(from = 0, to = 1, length.out = 101)))
 #' p2 <- predict(k, x)
 #' lines(x, p2$mean, col = "red")
 #' polygon(c(x, rev(x)), c(p2$mean - 2 * p2$stdev, rev(p2$mean + 2 * p2$stdev)),
 #'  border = NA, col = rgb(1, 0, 0, 0.2))
-update.Kriging <- function(object, newy, newX, ...) {
-
+update.Kriging <- function(object, y_u, X_u, refit=TRUE,...) {
     if (length(L <- list(...)) > 0) warnOnDots(L)
-    k <- kriging_model(object)
-    if (is.data.frame(newX)) newX = data.matrix(newX)
-    if (!is.matrix(newX)) newX <- matrix(newX, ncol = ncol(k$X))
-    if (is.data.frame(newy)) newy = data.matrix(newy)
-    if (!is.matrix(newy)) newy <- matrix(newy, ncol = ncol(k$y))
-    if (ncol(newX) != ncol(k$X))
-        stop("Object 'newX' must have ", ncol(k$X), " columns (instead of ",
-             ncol(newX), ")")
-    if (nrow(newy) != nrow(newX))
-        stop("Objects 'newX' and 'newy' must have the same number of rows.")
-
+    if (is.data.frame(X_u)) X_u = data.matrix(X_u)
+    if (!is.matrix(X_u)) X_u <- matrix(X_u, ncol = ncol(object$X()))
+    if (is.data.frame(y_u)) y_u = data.matrix(y_u)
+    if (!is.matrix(y_u)) y_u <- matrix(y_u, ncol = 1)
     ## Modify 'object' in the parent environment
-    kriging_update(object, newy, newX)
+    kriging_update(object, y_u, X_u, refit)
 
     invisible(NULL)
 }
 
 #' Save a Kriging Model to a file storage
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object An S3 Kriging object.
 #' @param filename File name to save in.
@@ -579,10 +628,9 @@ update.Kriging <- function(object, newy, newX, ...) {
 #' k <- Kriging(y, X, kernel = "matern3_2", objective="LMP")
 #' print(k)
 #'
-#' outfile = tempfile("k.h5") 
+#' outfile = tempfile("k.json") 
 #' save(k,outfile)
 save.Kriging <- function(object, filename, ...) {
-
     if (length(L <- list(...)) > 0) warnOnDots(L)
     if (!is.character(filename))
         stop("'filename' must be a string")
@@ -594,7 +642,7 @@ save.Kriging <- function(object, filename, ...) {
 
 #' Load a Kriging Model from a file storage
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param filename File name to load from.
 #' @param ... Not used.
@@ -612,7 +660,7 @@ save.Kriging <- function(object, filename, ...) {
 #' k <- Kriging(y, X, kernel = "matern3_2", objective="LMP")
 #' print(k)
 #'
-#' outfile = tempfile("k.h5")
+#' outfile = tempfile("k.json")
 #' save(k,outfile)
 #'
 #' print(load.Kriging(outfile)) 
@@ -620,18 +668,54 @@ load.Kriging <- function(filename, ...) {
     if (length(L <- list(...)) > 0) warnOnDots(L)
     if (!is.character(filename))
         stop("'filename' must be a string")
-    return( kriging_load(filename) )
+    return(classKriging( kriging_load(filename)))
+}
+
+#' Compute Covariance Matrix of Kriging Model
+#'
+#' @author Yann Richet \email{yann.richet@asnr.fr}
+#'
+#' @param object An S3 Kriging object.
+#' @param x1 Numeric matrix of input points.
+#' @param x2 Numeric matrix of input points.
+#' @param ... Not used.
+#' 
+#' @return A matrix of the covariance matrix of the Kriging model.
+#' 
+#' @method covMat Kriging
+#' @export
+#' @aliases covMat,Kriging,Kriging-method
+#' 
+#' @examples
+#' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
+#' set.seed(123)
+#' X <- as.matrix(runif(10))
+#' y <- f(X)
+#'
+#' k <- Kriging(y, X, kernel = "gauss")
+#' 
+#' x1 = runif(10)
+#' x2 = runif(10)
+#' 
+#' covMat(k, x1, x2)
+covMat.Kriging <- function(object, x1, x2, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
+    if (is.data.frame(x1)) x1 = data.matrix(x1)
+    if (is.data.frame(x2)) x2 = data.matrix(x2)
+    if (!is.matrix(x1)) x1 = matrix(x1, ncol = ncol(object$X()))
+    if (!is.matrix(x2)) x2 = matrix(x2, ncol = ncol(object$X()))
+    return(kriging_covMat(object, x1, x2))
 }
 
 #' Compute Log-Likelihood of Kriging Model
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object An S3 Kriging object.
 #' @param theta A numeric vector of (positive) range parameters at
 #'     which the log-likelihood will be evaluated.
-#' @param grad Logical. Should the function return the gradient?
-#' @param hess Logical. Should the function return Hessian?
+#' @param return_grad Logical. Should the function return the gradient?
+#' @param return_hess Logical. Should the function return Hessian?
 #' @param bench Logical. Should the function display benchmarking output?
 #' @param ... Not used.
 #'
@@ -657,13 +741,10 @@ load.Kriging <- function(filename, ...) {
 #' plot(t, ll(t), type = 'l')
 #' abline(v = k$theta(), col = "blue")
 logLikelihoodFun.Kriging <- function(object, theta,
-                                  grad = FALSE, hess = FALSE, bench=FALSE, ...) {
-    k <- kriging_model(object)
+                                  return_grad = FALSE, return_hess = FALSE, bench=FALSE, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
     if (is.data.frame(theta)) theta = data.matrix(theta)
-    if (!is.matrix(theta)) theta <- matrix(theta, ncol = ncol(k$X))
-    if (ncol(theta) != ncol(k$X))
-        stop("Input theta must have ", ncol(k$X), " columns (instead of ",
-             ncol(theta),")")
+    if (!is.matrix(theta)) theta <- matrix(theta, ncol = ncol(object$X()))
     out <- list(logLikelihood = matrix(NA, nrow = nrow(theta)),
                 logLikelihoodGrad = matrix(NA,nrow=nrow(theta),
                                            ncol = ncol(theta)),
@@ -671,13 +752,13 @@ logLikelihoodFun.Kriging <- function(object, theta,
                                                       ncol(theta))))
     for (i in 1:nrow(theta)) {
         ll <- kriging_logLikelihoodFun(object, theta[i, ],
-                                    grad = isTRUE(grad), hess = isTRUE(hess), bench = isTRUE(bench))
+                                    return_grad = isTRUE(return_grad), return_hess = isTRUE(return_hess), bench = isTRUE(bench))
         out$logLikelihood[i] <- ll$logLikelihood
-        if (isTRUE(grad)) out$logLikelihoodGrad[i, ] <- ll$logLikelihoodGrad
-        if (isTRUE(hess)) out$logLikelihoodHess[i, , ] <- ll$logLikelihoodHess
+        if (isTRUE(return_grad)) out$logLikelihoodGrad[i, ] <- ll$logLikelihoodGrad
+        if (isTRUE(return_hess)) out$logLikelihoodHess[i, , ] <- ll$logLikelihoodHess
     }
-    if (!isTRUE(grad)) out$logLikelihoodGrad <- NULL
-    if (!isTRUE(hess)) out$logLikelihoodHess <- NULL
+    if (!isTRUE(return_grad)) out$logLikelihoodGrad <- NULL
+    if (!isTRUE(return_hess)) out$logLikelihoodHess <- NULL
 
     return(out)
 }
@@ -685,7 +766,7 @@ logLikelihoodFun.Kriging <- function(object, theta,
 
 #' Get Log-Likelihood of Kriging Model
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object An S3 Kriging object.
 #' @param ... Not used.
@@ -708,6 +789,7 @@ logLikelihoodFun.Kriging <- function(object, theta,
 #'
 #' logLikelihood(k)
 logLikelihood.Kriging <- function(object, ...) {
+  if (length(L <- list(...)) > 0) warnOnDots(L)
   return(kriging_logLikelihood(object))
 }
 
@@ -720,13 +802,13 @@ logLikelihood.Kriging <- function(object, ...) {
 #' prediction of \eqn{y_i}{y[i]} based on the the observations \eqn{y_j}{y[j]}
 #' with \eqn{j \neq i}{j != i}.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object A \code{Kriging} object.
 #' @param theta A numeric vector of range parameters at which the LOO
 #'     will be evaluated.
 #'
-#' @param grad Logical. Should the gradient (w.r.t. \code{theta}) be
+#' @param return_grad Logical. Should the gradient (w.r.t. \code{theta}) be
 #'     returned?
 #' @param bench Logical. Should the function display benchmarking output
 #' @param ... Not used.
@@ -751,22 +833,19 @@ logLikelihood.Kriging <- function(object, ...) {
 #' t <-  seq(from = 0.001, to = 2, length.out = 101)
 #' plot(t, loo(t), type = "l")
 #' abline(v = k$theta(), col = "blue")
-leaveOneOutFun.Kriging <- function(object, theta, grad = FALSE, bench=FALSE, ...) {
-    k <- kriging_model(object)
+leaveOneOutFun.Kriging <- function(object, theta, return_grad = FALSE, bench=FALSE, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
     if (is.data.frame(theta)) theta = data.matrix(theta)
-    if (!is.matrix(theta)) theta <- matrix(theta,ncol=ncol(k$X))
-    if (ncol(theta) != ncol(k$X))
-        stop("Input theta must have ", ncol(k$X), " columns (instead of ",
-             ncol(theta),")")
+    if (!is.matrix(theta)) theta <- matrix(theta, ncol = ncol(object$X()))
     out <- list(leaveOneOut = matrix(NA, nrow = nrow(theta)),
                 leaveOneOutGrad = matrix(NA, nrow = nrow(theta),
                                          ncol = ncol(theta)))
     for (i in 1:nrow(theta)) {
-        loo <- kriging_leaveOneOutFun(object,theta[i,], isTRUE(grad), bench = isTRUE(bench))
+        loo <- kriging_leaveOneOutFun(object,theta[i,], isTRUE(return_grad), bench = isTRUE(bench))
         out$leaveOneOut[i] <- loo$leaveOneOut
-        if (isTRUE(grad)) out$leaveOneOutGrad[i, ] <- loo$leaveOneOutGrad
+        if (isTRUE(return_grad)) out$leaveOneOutGrad[i, ] <- loo$leaveOneOutGrad
     }
-    if (!isTRUE(grad)) out$leaveOneOutGrad <- NULL
+    if (!isTRUE(return_grad)) out$leaveOneOutGrad <- NULL
     return(out)
 }
 
@@ -777,7 +856,7 @@ leaveOneOutFun.Kriging <- function(object, theta, grad = FALSE, bench=FALSE, ...
 #' prediction of \eqn{y_i}{y[i]} based on the the observations \eqn{y_j}{y[j]}
 #' with \eqn{j \neq i}{j != i}.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object A \code{Kriging} object.
 #' @param theta A numeric vector of range parameters at which the LOO
@@ -794,14 +873,38 @@ leaveOneOutFun.Kriging <- function(object, theta, grad = FALSE, bench=FALSE, ...
 #' @examples
 #' f <- function(x) 1 - 1 / 2 * (sin(12 * x) / (1 + x) + 2 * cos(7 * x) * x^5 + 0.7)
 #' set.seed(123)
-#' X <- as.matrix(runif(10))
+#' X <- as.matrix(c(0.0, 0.25, 0.5, 0.75, 1.0))
 #' y <- f(X)
 #'
-#' k <- Kriging(y, X, kernel = "matern3_2", objective = "LOO", optim="BFGS")
+#' k <- Kriging(y, X, kernel = "matern3_2")
 #' print(k)
 #'
-#' leaveOneOutVec(k, k$theta())
+#' x <- as.matrix(seq(0, 1, , 101))
+#' p <- predict(k, x, TRUE, FALSE)
+#' 
+#' plot(f)
+#' points(X, y)
+#' lines(x, p$mean, col = 'blue')
+#' polygon(c(x, rev(x)), c(p$mean - 2 * p$stdev, rev(p$mean + 2 * p$stdev)),
+#'         border = NA, col = rgb(0, 0, 1, 0.2))
+#' 
+#' # Compute leave-one-out (no range re-estimate) on 2nd point
+#' X_no2 = X[-2,,drop=FALSE]
+#' y_no2 = f(X_no2)
+#' k_no2 = Kriging(y_no2, X_no2, "matern3_2", optim = "none", parameters = list(theta = k$theta()))
+#' print(k_no2)
+#' 
+#' p_no2 <- predict(k_no2, x, TRUE, FALSE)
+#' lines(x, p_no2$mean, col = 'red')
+#' polygon(c(x, rev(x)), c(p_no2$mean - 2 * p_no2$stdev, rev(p_no2$mean + 2 * p_no2$stdev)), 
+#'         border = NA, col = rgb(1, 0, 0, 0.2))
+#' 
+#' # Use leaveOneOutVec to get the same
+#' loov = k$leaveOneOutVec(matrix(k$theta()))
+#' points(X[2],loov$mean[2],col='red')
+#' lines(rep(X[2],2),loov$mean[2]+2*c(-loov$stdev[2],loov$stdev[2]),col='red')
 leaveOneOutVec.Kriging <- function(object, theta, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
     k <- kriging_model(object)
     if (!is.array(theta)) 
         stop("Input theta must be 1-dimensional")
@@ -814,7 +917,7 @@ leaveOneOutVec.Kriging <- function(object, theta, ...) {
 
 #' Get leaveOneOut of Kriging Model
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object An S3 Kriging object.
 #' @param ... Not used.
@@ -837,18 +940,19 @@ leaveOneOutVec.Kriging <- function(object, theta, ...) {
 #'
 #' leaveOneOut(k)
 leaveOneOut.Kriging <- function(object, ...) {
+  if (length(L <- list(...)) > 0) warnOnDots(L)
   return(kriging_leaveOneOut(object))
 }
 
 #' Compute the log-marginal posterior of a kriging model, using the
 #' prior XXXY.
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object S3 Kriging object.
 #' @param theta Numeric vector of correlation range parameters at
 #'     which the function is to be evaluated.
-#' @param grad Logical. Should the function return the gradient
+#' @param return_grad Logical. Should the function return the gradient
 #'     (w.r.t theta)?
 #' @param bench Logical. Should the function display benchmarking output?
 #' @param ... Not used.
@@ -879,29 +983,26 @@ leaveOneOut.Kriging <- function(object, ...) {
 #' t <- seq(from = 0.01, to = 2, length.out = 101)
 #' plot(t, lmp(t), type = "l")
 #' abline(v = k$theta(), col = "blue")
-logMargPostFun.Kriging <- function(object, theta, grad = FALSE, bench=FALSE, ...) {
-    k <- kriging_model(object)
+logMargPostFun.Kriging <- function(object, theta, return_grad = FALSE, bench=FALSE, ...) {
+    if (length(L <- list(...)) > 0) warnOnDots(L)
     if (is.data.frame(theta)) theta = data.matrix(theta)
-    if (!is.matrix(theta)) theta <- matrix(theta,ncol=ncol(k$X))
-    if (ncol(theta) != ncol(k$X))
-        stop("Input theta must have ", ncol(k$X), " columns (instead of ",
-             ncol(theta), ")")
+    if (!is.matrix(theta)) theta <- matrix(theta, ncol = ncol(object$X()))
     out <- list(logMargPost = matrix(NA, nrow = nrow(theta)),
                 logMargPostGrad = matrix(NA, nrow = nrow(theta),
                                          ncol = ncol(theta)))
     for (i in 1:nrow(theta)) {
-        lmp <- kriging_logMargPostFun(object, theta[i, ], grad = isTRUE(grad), bench = isTRUE(bench))
+        lmp <- kriging_logMargPostFun(object, theta[i, ], return_grad = isTRUE(return_grad), bench = isTRUE(bench))
         out$logMargPost[i] <- lmp$logMargPost
-        if (isTRUE(grad)) out$logMargPostGrad[i, ] <- lmp$logMargPostGrad
+        if (isTRUE(return_grad)) out$logMargPostGrad[i, ] <- lmp$logMargPostGrad
     }
-    if (!isTRUE(grad)) out$logMargPostGrad <- NULL
+    if (!isTRUE(return_grad)) out$logMargPostGrad <- NULL
     return(out)
 }
 
 
 #' Get logMargPost of Kriging Model
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object An S3 Kriging object.
 #' @param ... Not used.
@@ -924,13 +1025,14 @@ logMargPostFun.Kriging <- function(object, theta, grad = FALSE, bench=FALSE, ...
 #'
 #' logMargPost(k)
 logMargPost.Kriging <- function(object, ...) {
+  if (length(L <- list(...)) > 0) warnOnDots(L)
   return(kriging_logMargPost(object))
 }
 
 
 #' Duplicate a Kriging Model
 #'
-#' @author Yann Richet \email{yann.richet@irsn.fr}
+#' @author Yann Richet \email{yann.richet@asnr.fr}
 #'
 #' @param object An S3 Kriging object.
 #' @param ... Not used.
@@ -953,5 +1055,5 @@ logMargPost.Kriging <- function(object, ...) {
 #' print(copy(k))
 copy.Kriging <- function(object, ...) {
   if (length(L <- list(...)) > 0) warnOnDots(L)
-  return(kriging_copy(object))
+  return(classKriging(kriging_copy(object)))
 }
