@@ -58,7 +58,7 @@ void NoiseKriging::make_Cov(const std::string& covType) {
   } else
     throw std::invalid_argument("Unsupported covariance kernel: " + covType);
 
-  // arma::cout << "make_Cov done." << arma::endl;
+  // Rcpp::Rcout << "make_Cov done." << arma::endl;
 }
 
 LIBKRIGING_EXPORT arma::mat NoiseKriging::covMat(const arma::mat& X1, const arma::mat& X2) {
@@ -93,7 +93,7 @@ LIBKRIGING_EXPORT NoiseKriging::NoiseKriging(const arma::vec& y,
                                              const std::string& objective,
                                              const Parameters& parameters) {
   if (y.n_elem != X.n_rows)
-    throw std::runtime_error("Dimension of new data should be the same:\n X: (" + std::to_string(X.n_rows) + "x"
+    Rcpp::stop("Dimension of new data should be the same:\n X: (" + std::to_string(X.n_rows) + "x"
                              + std::to_string(X.n_cols) + "), y: (" + std::to_string(y.n_elem) + ")");
 
   make_Cov(covType);
@@ -166,7 +166,7 @@ double NoiseKriging::_logLikelihood(const arma::vec& _theta_sigma2,
                                     arma::vec* grad_out,
                                     NoiseKriging::KModel* model,
                                     std::map<std::string, double>* bench) const {
-  // arma::cout << " theta, sigma2: " << _theta_sigma2.t() << arma::endl;
+  // Rcpp::Rcout << " theta, sigma2: " << _theta_sigma2.t() << arma::endl;
 
   arma::uword d = m_X.n_cols;
   double _sigma2 = _theta_sigma2.at(d);
@@ -229,7 +229,7 @@ double NoiseKriging::_logLikelihood(const arma::vec& _theta_sigma2,
     } else
       (*grad_out).at(d) = 0;  // if sigma2 is defined & fixed by user
 
-    // arma::cout << " grad_out:" << *grad_out << arma::endl;
+    // Rcpp::Rcout << " grad_out:" << *grad_out << arma::endl;
   }
   return ll;
 }
@@ -251,8 +251,9 @@ LIBKRIGING_EXPORT std::tuple<double, arma::vec> NoiseKriging::logLikelihoodFun(c
     size_t num = 0;
     for (auto& kv : bench)
       num = std::max(kv.first.size(), num);
-    for (auto& kv : bench)
-      arma::cout << "| " << Bench::pad(kv.first, num, ' ') << " | " << kv.second << " |" << arma::endl;
+    for (auto& kv : bench) {
+      Rcpp::Rcout << "| " << Bench::pad(kv.first, num, ' ') << " | " << kv.second << " |" << arma::endl;
+    }
 
   } else {
     if (_grad) {
@@ -300,13 +301,13 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
     if (Optim::reparametrize) {
       fit_ofn = CacheFunction([this](const arma::vec& _gamma, arma::vec* grad_out, NoiseKriging::KModel* km_data) {
         // Change variable for opt: . -> 1/exp(.)
-        // DEBUG: if (Optim::log_level>3) arma::cout << "> gamma: " << _gamma << arma::endl;
+        // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "> gamma: " << _gamma << arma::endl;
         const arma::vec _theta_sigma2 = Optim::reparam_from(_gamma);
-        // DEBUG: if (Optim::log_level>3) arma::cout << "> theta_alpha: " << _theta_sigma2 << arma::endl;
+        // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "> theta_alpha: " << _theta_sigma2 << arma::endl;
         double ll = this->_logLikelihood(_theta_sigma2, grad_out, km_data, nullptr);
-        // DEBUG: if (Optim::log_level>3) arma::cout << "  > ll: " << ll << arma::endl;
+        // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "  > ll: " << ll << arma::endl;
         if (grad_out != nullptr) {
-          // DEBUG: if (Optim::log_level>3) arma::cout << "  > grad ll: " << grad_out << arma::endl;
+          // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "  > grad ll: " << grad_out << arma::endl;
           *grad_out = -Optim::reparam_from_deriv(_theta_sigma2, *grad_out);
         }
         return -ll;
@@ -314,11 +315,11 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
     } else {
       fit_ofn = CacheFunction([this](const arma::vec& _gamma, arma::vec* grad_out, NoiseKriging::KModel* km_data) {
         const arma::vec _theta_sigma2 = _gamma;
-        // DEBUG: if (Optim::log_level>3) arma::cout << "> theta_alpha: " << _theta_sigma2 << arma::endl;
+        // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "> theta_alpha: " << _theta_sigma2 << arma::endl;
         double ll = this->_logLikelihood(_theta_sigma2, grad_out, km_data, nullptr);
-        // DEBUG: if (Optim::log_level>3) arma::cout << "  > ll: " << ll << arma::endl;
+        // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "  > ll: " << ll << arma::endl;
         if (grad_out != nullptr) {
-          // DEBUG: if (Optim::log_level>3) arma::cout << "  > grad ll: " << grad_out << arma::endl;
+          // DEBUG: if (Optim::log_level>3) Rcpp::Rcout << "  > grad ll: " << grad_out << arma::endl;
           *grad_out = -*grad_out;
         }
         return -ll;
@@ -375,8 +376,8 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
   // Define regression matrix
   m_regmodel = regmodel;
   m_F = Trend::regressionModelMatrix(regmodel, m_X);
-  m_est_beta = (m_regmodel != Trend::RegressionModel::None);
-  if ((parameters.beta.has_value())
+  m_est_beta = parameters.is_beta_estim && (m_regmodel != Trend::RegressionModel::None);
+  if (!m_est_beta && parameters.beta.has_value()
       && parameters.beta.value().n_elem > 0) {  // Then force beta to be fixed (not estimated, no variance)
     m_est_beta = false;
     m_beta = parameters.beta.value();
@@ -393,15 +394,15 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
     if (m_normalize)
       theta0.each_row() /= scaleX;
     if (theta0.n_cols != d)
-      throw std::runtime_error("Dimension of theta should be nx" + std::to_string(d) + " instead of "
+      Rcpp::stop("Dimension of theta should be nx" + std::to_string(d) + " instead of "
                                + std::to_string(theta0.n_rows) + "x" + std::to_string(theta0.n_cols));
   }
 
   if (optim == "none") {  // just keep given theta, no optimisation of ll (but estim beta still possible)
     if (!parameters.theta.has_value())
-      throw std::runtime_error("Theta should be given (1x" + std::to_string(d) + ") matrix, when optim=none");
+      Rcpp::stop("Theta should be given (1x" + std::to_string(d) + ") matrix, when optim=none");
     if (!parameters.sigma2.has_value())
-      throw std::runtime_error("Sigma2 should be given, when optim=none");
+      Rcpp::stop("Sigma2 should be given, when optim=none");
 
     m_theta = trans(theta0.row(0));
     m_est_theta = false;
@@ -446,11 +447,11 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
     if (Optim::variogram_bounds_heuristic) {
       // dy2 /= arma::var(m_y);
       arma::vec dy2dX2_slope = dy2 / arma::sum(m_dX % m_dX, 0).t();
-      // arma::cout << "dy2dX_slope:" << dy2dX_slope << arma::endl;
+      // Rcpp::Rcout << "dy2dX_slope:" << dy2dX_slope << arma::endl;
       dy2dX2_slope.replace(arma::datum::nan, 0.0);  // we are not interested in same points where dX=0, and dy=0
       arma::vec w = dy2dX2_slope / sum(dy2dX2_slope);
       arma::mat steepest_dX_mean = arma::abs(m_dX) * w;
-      // arma::cout << "steepest_dX_mean:" << steepest_dX_mean << arma::endl;
+      // Rcpp::Rcout << "steepest_dX_mean:" << steepest_dX_mean << arma::endl;
 
       theta_lower = arma::max(theta_lower, Optim::theta_lower_factor * steepest_dX_mean);
       // no, only relevant for inf bound: theta_upper = arma::min(theta_upper, Optim::theta_upper_factor *
@@ -458,8 +459,8 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
       theta_lower = arma::min(theta_lower, theta_upper);
       theta_upper = arma::max(theta_lower, theta_upper);
     }
-    // arma::cout << "theta_lower:" << theta_lower << arma::endl;
-    // arma::cout << "theta_upper:" << theta_upper << arma::endl;
+    // Rcpp::Rcout << "theta_lower:" << theta_lower << arma::endl;
+    // Rcpp::Rcout << "theta_upper:" << theta_upper << arma::endl;
 
     int multistart = 1;
     try {
@@ -481,7 +482,7 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
     } else {
       theta0 = theta0_rand;
     }
-    // arma::cout << "theta0:" << theta0 << arma::endl;
+    // Rcpp::Rcout << "theta0:" << theta0 << arma::endl;
 
     arma::vec dX2 = arma::sum(m_dX % m_dX, 0).t();
 
@@ -497,7 +498,7 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
     } else {
       sigma20 = sigma2_lower + (sigma2_upper - sigma2_lower) * Random::randu_vec(theta0.n_rows);
     }
-    // arma::cout << "sigma20:" << sigma20 << arma::endl;
+    // Rcpp::Rcout << "sigma20:" << sigma20 << arma::endl;
 
     arma::vec gamma_lower = arma::vec(d + 1);
     gamma_lower.head(d) = theta_lower;
@@ -525,18 +526,18 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
       gamma_upper = arma::max(gamma_tmp, gamma_upper);
 
       if (Optim::log_level > 0) {
-        arma::cout << "BFGS:" << arma::endl;
-        arma::cout << "  max iterations: " << Optim::max_iteration << arma::endl;
-        arma::cout << "  null gradient tolerance: " << Optim::gradient_tolerance << arma::endl;
-        arma::cout << "  constant objective tolerance: " << Optim::objective_rel_tolerance << arma::endl;
-        arma::cout << "  reparametrize: " << Optim::reparametrize << arma::endl;
-        arma::cout << "  normalize: " << m_normalize << arma::endl;
-        arma::cout << "  lower_bounds: " << theta_lower.t() << "";
-        arma::cout << "                " << sigma2_lower << arma::endl;
-        arma::cout << "  upper_bounds: " << theta_upper.t() << "";
-        arma::cout << "                " << sigma2_upper << arma::endl;
-        arma::cout << "  start_point: " << theta0.row(i % multistart) << "";
-        arma::cout << "               " << sigma20[i % sigma20.n_elem] << arma::endl;
+        Rcpp::Rcout << "BFGS:" << arma::endl;
+        Rcpp::Rcout << "  max iterations: " << Optim::max_iteration << arma::endl;
+        Rcpp::Rcout << "  null gradient tolerance: " << Optim::gradient_tolerance << arma::endl;
+        Rcpp::Rcout << "  constant objective tolerance: " << Optim::objective_rel_tolerance << arma::endl;
+        Rcpp::Rcout << "  reparametrize: " << Optim::reparametrize << arma::endl;
+        Rcpp::Rcout << "  normalize: " << m_normalize << arma::endl;
+        Rcpp::Rcout << "  lower_bounds: " << theta_lower.t() << "";
+        Rcpp::Rcout << "                " << sigma2_lower << arma::endl;
+        Rcpp::Rcout << "  upper_bounds: " << theta_upper.t() << "";
+        Rcpp::Rcout << "                " << sigma2_upper << arma::endl;
+        Rcpp::Rcout << "  start_point: " << theta0.row(i % multistart) << "";
+        Rcpp::Rcout << "               " << sigma20[i % sigma20.n_elem] << arma::endl;
       }
 
       m_est_sigma2 = parameters.is_sigma2_estim;
@@ -572,14 +573,14 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
             bounds_type.memptr());
 
         if (Optim::log_level > 0) {
-          arma::cout << "     iterations: " << result.num_iters << arma::endl;
-          arma::cout << "     status: " << result.task << arma::endl;
+          Rcpp::Rcout << "     iterations: " << result.num_iters << arma::endl;
+          Rcpp::Rcout << "     status: " << result.task << arma::endl;
           if (Optim::reparametrize) {
-            arma::cout << "     start_point: " << Optim::reparam_from(gamma_0).t() << " ";
-            arma::cout << "     solution: " << Optim::reparam_from(gamma_tmp).t() << " ";
+            Rcpp::Rcout << "     start_point: " << Optim::reparam_from(gamma_0).t() << " ";
+            Rcpp::Rcout << "     solution: " << Optim::reparam_from(gamma_tmp).t() << " ";
           } else {
-            arma::cout << "     start_point: " << gamma_0.t() << " ";
-            arma::cout << "     solution: " << gamma_tmp.t() << " ";
+            Rcpp::Rcout << "     start_point: " << gamma_0.t() << " ";
+            Rcpp::Rcout << "     solution: " << gamma_tmp.t() << " ";
           }
         }
 
@@ -617,8 +618,9 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
 
           retry++;
         } else {
-          if (Optim::log_level > 1)
+          if (Optim::log_level > 1) {
             result.print();
+          }
           break;
         }
       }
@@ -627,11 +629,12 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
       double min_ofn_tmp = fit_ofn(best_gamma, nullptr, &m);
 
       if (Optim::log_level > 0) {
-        arma::cout << "  best objective: " << min_ofn_tmp << arma::endl;
-        if (Optim::reparametrize)
-          arma::cout << "  best solution: " << Optim::reparam_from(best_gamma).t() << " ";
-        else
-          arma::cout << "  best solution: " << best_gamma.t() << "";
+        Rcpp::Rcout << "  best objective: " << min_ofn_tmp << arma::endl;
+        if (Optim::reparametrize) {
+          Rcpp::Rcout << "  best solution: " << Optim::reparam_from(best_gamma).t() << " ";
+        } else {
+          Rcpp::Rcout << "  best solution: " << best_gamma.t() << "";
+        }
       }
 
       if (min_ofn_tmp < min_ofn) {
@@ -662,9 +665,9 @@ LIBKRIGING_EXPORT void NoiseKriging::fit(const arma::vec& y,
       }
     }
   } else
-    throw std::runtime_error("Unsupported optim: " + optim + " (supported are: none, BFGS[#])");
+    Rcpp::stop("Unsupported optim: " + optim + " (supported are: none, BFGS[#])");
 
-  // arma::cout << "theta:" << m_theta << arma::endl;
+  // Rcpp::Rcout << "theta:" << m_theta << arma::endl;
 }
 
 /** Compute the prediction for given points X'
@@ -680,7 +683,7 @@ NoiseKriging::predict(const arma::mat& X_n, bool return_stdev, bool return_cov,
   arma::uword n_o = m_X.n_rows;
   arma::uword d = m_X.n_cols;
   if (X_n.n_cols != d)
-    throw std::runtime_error("Predict locations have wrong dimension: " + std::to_string(X_n.n_cols) + " instead of "
+    Rcpp::stop("Predict locations have wrong dimension: " + std::to_string(X_n.n_cols) + " instead of "
                              + std::to_string(d));
 
   arma::vec yhat_n(n_n);
@@ -855,11 +858,11 @@ LIBKRIGING_EXPORT arma::mat NoiseKriging::simulate(const int nsim,
   arma::uword n_o = m_X.n_rows;
   arma::uword d = m_X.n_cols;
   if (X_n.n_cols != d)
-    throw std::runtime_error("Simulate locations have wrong dimension: " + std::to_string(X_n.n_cols) + " instead of "
+    Rcpp::stop("Simulate locations have wrong dimension: " + std::to_string(X_n.n_cols) + " instead of "
                              + std::to_string(d));
 
   if (with_noise.n_elem > 1 && with_noise.n_elem != n_n)
-    throw std::runtime_error("Noise vector should have same length as X_n: " + std::to_string(with_noise.n_elem)
+    Rcpp::stop("Noise vector should have same length as X_n: " + std::to_string(with_noise.n_elem)
                              + " instead of " + std::to_string(n_n) + " (or 0 if no noise)");
 
   arma::mat Xn_o = trans(m_X);  // already normalized if needed
@@ -977,22 +980,22 @@ LIBKRIGING_EXPORT arma::mat NoiseKriging::update_simulate(const arma::vec& y_u,
                                                           const arma::vec& noise_u,
                                                           const arma::mat& X_u) {
   if (y_u.n_elem != X_u.n_rows)
-    throw std::runtime_error("Dimension of new data should be the same:\n X: (" + std::to_string(X_u.n_rows) + "x"
+    Rcpp::stop("Dimension of new data should be the same:\n X: (" + std::to_string(X_u.n_rows) + "x"
                              + std::to_string(X_u.n_cols) + "), y: (" + std::to_string(y_u.n_elem) + ")");
 
   if (X_u.n_cols != m_X.n_cols)
-    throw std::runtime_error("Dimension of new data should be the same:\n X: (...x" + std::to_string(m_X.n_cols)
+    Rcpp::stop("Dimension of new data should be the same:\n X: (...x" + std::to_string(m_X.n_cols)
                              + "), new X: (...x" + std::to_string(X_u.n_cols) + ")");
 
   if (noise_u.n_elem != X_u.n_rows)
-    throw std::runtime_error("Noise vector should have same length as X_u: " + std::to_string(noise_u.n_elem)
+    Rcpp::stop("Noise vector should have same length as X_u: " + std::to_string(noise_u.n_elem)
                              + " instead of " + std::to_string(X_u.n_rows));
 
   if (lastsim_y_n.is_empty() || lastsim_y_n.n_rows == 0)
-    throw std::runtime_error("No previous simulation data available");
+    Rcpp::stop("No previous simulation data available");
 
   if (lastsim_Xn_n.n_rows != X_u.n_cols)
-    throw std::runtime_error("Dimension of new data should be the same:\n X: (...x" + std::to_string(X_u.n_cols)
+    Rcpp::stop("Dimension of new data should be the same:\n X: (...x" + std::to_string(X_u.n_cols)
                              + "), last sim X: (...x" + std::to_string(lastsim_Xn_n.n_rows) + ")");
 
   arma::uword n_n = lastsim_Xn_n.n_cols;
@@ -1142,15 +1145,15 @@ LIBKRIGING_EXPORT void NoiseKriging::update(const arma::vec& y_u,
                                             const arma::mat& X_u,
                                             const bool refit) {
   if (y_u.n_elem != X_u.n_rows)
-    throw std::runtime_error("Dimension of new data should be the same:\n X: (" + std::to_string(X_u.n_rows) + "x"
+    Rcpp::stop("Dimension of new data should be the same:\n X: (" + std::to_string(X_u.n_rows) + "x"
                              + std::to_string(X_u.n_cols) + "), y: (" + std::to_string(y_u.n_elem) + ")");
 
   if (noise_u.n_elem != y_u.n_elem)
-    throw std::runtime_error("Dimension of new data should be the same:\n noise: (" + std::to_string(noise_u.n_elem)
+    Rcpp::stop("Dimension of new data should be the same:\n noise: (" + std::to_string(noise_u.n_elem)
                              + "), y: (" + std::to_string(y_u.n_elem) + ")");
 
   if (X_u.n_cols != m_X.n_cols)
-    throw std::runtime_error("Dimension of new data should be the same:\n X: (...x" + std::to_string(m_X.n_cols)
+    Rcpp::stop("Dimension of new data should be the same:\n X: (...x" + std::to_string(m_X.n_cols)
                              + "), new X: (...x" + std::to_string(X_u.n_cols) + ")");
 
   // rebuild starting parameters
@@ -1295,11 +1298,11 @@ NoiseKriging NoiseKriging::load(const std::string filename) {
 
   uint32_t version = j["version"].template get<uint32_t>();
   if (version != 2) {
-    throw std::runtime_error(asString("Bad version to load from '", filename, "'; found ", version, ", requires 2"));
+    Rcpp::stop(asString("Bad version to load from '", filename, "'; found ", version, ", requires 2"));
   }
   std::string content = j["content"].template get<std::string>();
   if (content != "NoiseKriging") {
-    throw std::runtime_error(
+    Rcpp::stop(
         asString("Bad content to load from '", filename, "'; found '", content, "', requires 'NoiseKriging'"));
   }
 
